/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.core.naming;

/** To use the ContextContainer interface */
import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.api.Wrapper;
import org.objectweb.util.browser.core.api.ContextContainer;
import org.objectweb.util.browser.core.api.Decoder;

/**
 * Interceptor for ContextContainer
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>
 * @version 0.1
 */

public class ContextContainerDecoderManager 
    implements ContextContainer {

    // ==================================================================
    //
    // Internal states.
    //
    // ==================================================================

    /** The decoder. */
    protected Decoder decoder_;

    /** The delegate */
    protected ContextContainer delegate_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Creates an Entry.
     *
     * @param value The value of the entry to create.
     * @param id The id of the name of the entry to create.
     *
     * @return A new Entry.
     *
     */
    protected Entry createEntry(Object value, String id) {
        return new DefaultEntry(value, new DefaultName(id), this);
    }

    // ==================================================================
    //
    // Public methods for interface Context.
    //
    // ==================================================================

    /**
     * Returns the entry identify by the given name
     * @param name The key of the entry
     * @return The associated entry or null.
     */
    public Entry getLocalEntry(String name) {
        Entry e = delegate_.getLocalEntry(name);
        return (e != null) ? createEntry(decoder_.decode(e.getValue()),e.getName().toString()): null;
    }

    /**
     * Returns the value of the entry identify by the given name
     * @param name The key of the entry
     * @return The value of the associated entry or null.
     */
    public Object getLocalValue(String name){
        Entry entry = getLocalEntry(name);
        return entry!=null ? entry.getValue() : null;    
    }
    
    /**
     * Returns an array containing the entries contained by the target context.
     * @return A new array of entry.
     */
    public Entry[] getEntries(){
        Entry[] entries = delegate_.getEntries();
        Entry[] decodedEntries = new Entry[entries.length];
        for (int i = 0; i < entries.length; i++) {
            if (entries[i] != null) 
                decodedEntries[i] = createEntry(decoder_.decode(entries[i].getValue()),entries[i].getName().toString());
		}
        return decodedEntries; 
    }

    // ==================================================================
    //
    // Additional public methods for Wrapper interface.
    //
    // ==================================================================

    /**
     * Fix the wrapped object
     *
     * @param object The object to wrap
     *
     */
    public void setWrapped(Object object) {
        //((Wrapper)delegate_).setWrapped(decoder_.decode(object));
        ((Wrapper) delegate_).setWrapped(object);
    }

    /**
     * Gets the wrapped object
     *
     * @return The wrapped object
     *
     */
    public Object getWrapped() {
        return ((Wrapper) delegate_).getWrapped();
    }

    // ==================================================================
    //
    // Additional public methods for class DefaultContextContainer.
    //
    // ==================================================================

    /**
     * Adds an object into the container.
     * If an entry with the same id already exists in the container, this one is replaced by the new value. 
     * It decodes the given object and contacts the delegate object to add this entry
     *
     * This method does not modify the given id and the given object, But it modifies the target context.
     *
     * @param id The name of the entry to be retrieved.
     * @param object The object to add
     * 
     */
    public void addEntry(String id, Object object) {
        delegate_.addEntry(id, decoder_.decode(object));
    }

    /**
     * Removes the object corresponding to the given id from the context
     * This method modifies the target context.
     * @param id The name of the entry to be removed.
     */
    public void removeEntry(String id) {
        delegate_.removeEntry(id);
    }

    // ==================================================================
    //
    // Additional public methods.
    //
    // ==================================================================

    /**
     * Fixes the delegate of the decoder
     */
    public void setDelegate(ContextContainer context) {
        delegate_ = context;
    }

    /**
     * Fixes the decoder
     */
    public void setDecoder(Decoder decoder) {
        decoder_ = decoder;
    }
}
