package org.coach.tracing.server.viewer;

import javax.swing.*;
import javax.swing.tree.*;
import java.awt.*;
import java.util.*;

public class ThreadNode extends DefaultMutableTreeNode {
    // Maintain all identities in a linear map for fast lookup.
    private static HashMap threadMap = new HashMap();
    // display order list
    private static LinkedList threadList = new LinkedList();

    private static ThreadNode root = new ThreadNode("");

    // Array of currently visible identies.
    private static ThreadNode[] visibles;

    private String id;
        
    private boolean isExpanded = true;
    private boolean isCollapsed = false;
    private boolean isHidden = false;

    /**
     * Search for an Identity object in the hierarchy tree.
     * If the key is not in the tree the identity is obtained from the database.
     */    
    public static ThreadNode get(String processId, String threadId) {
        ThreadNode n = (ThreadNode)threadMap.get(threadId);
        if (n == null) {
            n = new ThreadNode(threadId);
            n.isExpanded = false;
            
            // Obtain the parent node and add the new identity as its child.
            ThreadNode parent = (ThreadNode)threadMap.get(processId);
            if (parent == null) {
                parent = new ThreadNode(processId);
                parent.isExpanded = true;
                threadMap.put(processId, parent);
                threadList.add(parent);
                root.add(parent);
            }
            
            parent.add(n);
            
            // Update the identity map for fast lookup.
            threadMap.put(threadId, n);
            threadList.add(n);
        }
        return n;
    }

    public static void clear() {
        threadMap.clear();
        threadList.clear();
        root = new ThreadNode("");
        visibles = null;
    }
    
    public static ThreadNode getRootNode() {
        return root;
    }
    
    public static ThreadNode[] getVisibles() {
        if (visibles == null) {       
            LinkedList s = new LinkedList();
            Iterator it = threadList.iterator();
            while(it.hasNext()) {
                ThreadNode n = (ThreadNode)it.next();
                // Iterate over all leaf object nodes.
                if (n.isVisible()) {
                    s.add(n);
                }
            }
            visibles = new ThreadNode[s.size()];
            s.toArray(visibles);
        }
        return visibles;
    }
    
    public static void moveLeft(ThreadNode n) {
        int idx = threadList.indexOf(n);
        if (idx > 0) {
            threadList.remove(idx);
            threadList.add(idx - 1, n);
            if (!((ThreadNode)threadList.get(idx)).isVisible()) {
                // The previous node was not visible, so move again.
                // Otherwise the user does not see the move on his screen.
                moveLeft(n);
            }
            visibles = null;
        }
    }

    public static void moveRight(ThreadNode n) {
        int idx = threadList.indexOf(n);
        if (idx >= 0 && idx < threadList.size() - 1) {
            threadList.remove(idx);
            threadList.add(idx + 1, n);
            if (!((ThreadNode)threadList.get(idx)).isVisible()) {
                // The previous node was not visible, so move again.
                // Otherwise the user does not see the move on his screen.
                moveRight(n);
            }
            visibles = null;
        }
    }

    public static void unhideAll() {
        Iterator it = threadList.iterator();
        while(it.hasNext()) {
            ThreadNode n = (ThreadNode)it.next();
            n.isHidden = false;
        }
    }

    /**
     * private constructor. The identity tree structure is created through the
     * static get operation.
     */    
    private ThreadNode(String id) {
        this.id = id;
        setUserObject(this);
        // invalidate the list of visible identities.
        visibles = null;
    }
    
    public String getName() {
        return id;
    }

    public String toString() {
        return id + " " + isVisible();
    }

    public String getShortName() {
        String result = null;
        //trim from end to rightmost dot.
        int index = id.lastIndexOf("_");
        if (index > 0) {
            result = id.substring(0, index);
            index = result.lastIndexOf("/");
            if (index > 0) {
                result = result.substring(index + 1);
            }
            return result;
        }
        return result;
    }

    // If a node is hidden, all its children are hidden too.        
    public void setHidden(boolean hide) {
        this.isHidden = hide;
        int cnt = getChildCount();
        for (int i = 0; i < cnt; i++) {
            ThreadNode mo = (ThreadNode)getChildAt(i);
            mo.setHidden(hide);
        } 
        // invalidate the list of visible identities.
        visibles = null;
    }
    
    public void expand() {
        int cnt = getChildCount();
        for (int i = 0; i < cnt; i++) {
            ThreadNode n = (ThreadNode)getChildAt(i);
            n.collapseChildren();
        }
        isExpanded = true;
        isCollapsed = false;
        // invalidate the list of visible identities.
        visibles = null;
    }

    /**
     * When a node is collapsed, its parent becomes visible.
     */
    public void collapse() {
        if (!isRoot()) {
            ThreadNode p = (ThreadNode)getParent();
            p.collapseChildren();             
        }        
        // invalidate the list of visible identities.
        visibles = null;
    }

    /**
     * Collapse the whole subtree.
     */
    private void collapseChildren() {
        if (!isRoot()) {
            Enumeration e = depthFirstEnumeration();
            while(e.hasMoreElements()) {
                ThreadNode n = (ThreadNode)e.nextElement();        
                n.isCollapsed = true;
            }
            isCollapsed = false;
            isExpanded = false;
        }
    }

    public boolean isVisible() {
        if (isRoot() || isHidden || isCollapsed) {
            return false;
        }
        
        if (!isLeaf() && isExpanded) {
            return false;
        }

        return true;
    }
    
    /**
     * Returns the node that is used to map the GUI for this node.
     */
    public ThreadNode getVisible() {
        if (isRoot()) {
            return null;
        }
        
        if (isVisible()) {
            return this;
        } else {
            ThreadNode p = (ThreadNode)getParent();
            return p.getVisible();
        }
    }
       
    public boolean isHidden() {
        if (isRoot()) {
            return false;
        }
        
        if (isHidden) {
            return true;
        }
            
        return ((ThreadNode)getParent()).isHidden();
    }       
}