/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.ccm.Deployment;



import org.objectweb.ccm.Containers.MetaInformation.HomeType;
import org.objectweb.ccm.Containers.MetaInformation.HomeInstance;
/**
 * The goal of this class is to provide a 
 * temporary way to produce system component
 * avoiding the use of deployment architecture.
 * It will be particulary use for producing deployment
 * component such as DCI or AssemblyFactoryManager.
 *
 * @author <a href="mailto:frederic.briclet@lifl.fr">Briclet Frederic</a>
 * @author <a href="mailto:Fabien.Hameau@lifl.fr">Hameau Fabien</a>
 * @version 0.1
 */

public class DeploymentComponentFactory {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    private static SystemHomeManagerImpl home_manager_ = 
        new SystemHomeManagerImpl();
    
    private static org.objectweb.ccm.Containers.PCA rootPCA_ = 
        new org.objectweb.ccm.Containers.RootPCAImpl(home_manager_);

    // Adding a reference to the home type MI managed by     
    // this container 
    protected HomeType home_type_;

    /**
     * Used by the computeFullHomeServantClassName method in order
     * to compute full home servant class names.
     */
    static private org.objectweb.ccm.visitorIDL3.java.JavaTranslator
        javaTranslator  = new org.objectweb.ccm.visitorIDL3.java.JavaTranslator();

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================
    
    /* private DeploymentComponentFactory(){        
       }*/
    
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    /**
     * Trace an error.
     */
    protected static void
    trace_error(String method,
                String msg,
                Throwable exc)
    {
        System.err.println("DeploymentComponentFactory::" + method + ": " + msg);
        if(exc != null)
            exc.printStackTrace();
    }


    /**
     * Compute the full home servant class name associated
     * to a home executor class.
     *
     * @param homeExecutorClass The home executor class.
     *
     * @return The full home servant class name, or null if not found.
     */
    protected static String
    computeFullHomeServantClassName(Class homeExecutorClass)
    {
        // If homeExecutorClass is null then returns null.
        if(homeExecutorClass == null)
            return null;

        // The full class name with the package name.
        String fullClassName = homeExecutorClass.getName();

        // The short class name without the package name.
        int idx = fullClassName.lastIndexOf(".") + 1;
        String shortClassName = fullClassName.substring(idx);

        // If the short class name is prefixed CCM_ then
        // returns the full home servant class name.
        if(shortClassName.startsWith("CCM_"))
            return fullClassName.substring(0, idx)
                 + javaTranslator.checkKeywords(shortClassName.substring(4))
                 + "CCM";

        // Find the CCM home executor interface into implemented interfaces.
        //
        Class[] itfs = homeExecutorClass.getInterfaces();

    // Required to navigate in the reverse order
        // else interface CCM_*Operations will be found
        // before interface CCM_*
        for(int i=itfs.length-1; i>=0; i--)
        {
             String className = computeFullHomeServantClassName(itfs[i]);
             if(className != null)
                 return className;
        }

    // Find the CCM home executor interface into the super class.
        String className = computeFullHomeServantClassName(homeExecutorClass.getSuperclass());
        if(className != null)
            return className;

        // Returns null as the home servant class was not found.
        return null;
    }


     /**
     * Creates a home executor.
     *
     * @param entrypt The entry point in the archive.
     *
     * @return The created home executor.
     *
     * @exception org.omg.Components.Deployment.ImplEntryPointNotFound
     *            Thrown if the home executor entry point is invalid.
     */
    protected static org.omg.Components.HomeExecutorBase
    create_home_executor(java.lang.String entrypt)
    throws org.omg.Components.Deployment.ImplEntryPointNotFound,
           org.omg.Components.Deployment.InstallationFailure
    {
        try
        {
            // Call the static home executor 'entrypt' method.
            java.lang.Object object =
                TheURLClassLoader.getURLClassLoader()
                .callStaticClassMethod(entrypt, new Object[0]);

            // Check that the result is a CCMHome executor.
            return (org.omg.Components.HomeExecutorBase)object;
        }
        catch(java.lang.ClassNotFoundException exc)
        {
            trace_error("install_home",
                        "Class '" + exc.getMessage() + "' not found for home executor entry point '"
                        + entrypt + "'!!!",
                        null);
            throw new org.omg.Components.Deployment.ImplEntryPointNotFound();
        }
        catch(java.lang.NoSuchMethodException exc)
        {
            trace_error("install_home",
                        "Home executor entry point 'public static org.omg.Components.HomeExecutorBase "
                        + entrypt + "();' not found, check the home executor entry point signature!!!",
                         null);
            throw new org.omg.Components.Deployment.ImplEntryPointNotFound();
        }
        catch(java.lang.reflect.InvocationTargetException exc)
        {
            trace_error("install_home",
                        "Exception thrown by home executor entry point '" + entrypt + "'!!!",
                        exc.getTargetException());
            throw new org.omg.Components.Deployment.InstallationFailure(
                          org.objectweb.ccm.Deployment.ExceptionThrownByHomeExecutorEntryPoint.value
                  );
        }
        catch(java.lang.IllegalAccessException exc)
        {
            trace_error("install_home",
                        "Illegal access by home executor entry point '" + entrypt + "'!!!",
                        exc);
            throw new org.omg.Components.Deployment.InstallationFailure(
                          org.objectweb.ccm.Deployment.IllegalAccessByHomeExecutorEntryPoint.value
                  );
        }
        catch(java.lang.ClassCastException exc)
        {
            trace_error("install_home",
                        "Home executor entry point '" + entrypt +
                        "' does not return org.omg.Components.HomeExecutorBase," +
                        " check the home executor entry point signature!!!",
                        null);
            throw new org.omg.Components.Deployment.ImplEntryPointNotFound();
        }
    }


    protected String retrieve_MI_name(String className)
    {
        try
            {
                // isolate names
                String tmpName = className.substring(0, className.lastIndexOf("Impl."));
                // get the IDL name of home
                String IDLName = tmpName.substring(tmpName.lastIndexOf(".") + 1);
                // get the prefix for concerned nale
                String prefix = tmpName.substring(0,tmpName.lastIndexOf("."));
                prefix = tmpName.substring(0,prefix.lastIndexOf(".") + 1);
                // compute the full name
                if (IDLName != null)
                    {
                       // System.err.println("### Debug : get " + prefix + IDLName);
                        return prefix + IDLName;
                    }
            }
        // problem during name computation
        catch(Exception e)
            {
               // trace_error("retrieve_MI_name",
               //   "error when computing IDL name for MetaInformation", null);
            }
        return null;
    }
    
    /**
     * Install a home for a archive already load in memory
     * that is the case of system archive.
     *
     * @param entryPoint the entryPoint to instantiate the home
     */    
    public org.omg.Components.CCMHome 
    install_home(String entrypt)
    {
        try {
            // create the home executor.
            org.omg.Components.HomeExecutorBase home =
                create_home_executor(entrypt);           
            
            String clazz_name = computeFullHomeServantClassName(home.getClass());
            if (clazz_name == null) 
                {
                    trace_error("install_home",
                                "Home servant class for " + home.getClass().getName() + " not found!!!",
                                null);
                    throw new org.omg.Components.Deployment.
                        InstallationFailure(org.objectweb.ccm.Deployment.HomeServantClassNotFound.value);
                }

            // Instantiate the executor.
            org.objectweb.ccm.Containers.HomeExecutorBase home_exec =
                (org.objectweb.ccm.Containers.HomeExecutorBase)
                create_home_executor(clazz_name + "._create_home_executor");
            
            home_exec._delegate(home);
            
            // create the home servant and set infos.
            org.objectweb.ccm.Containers.HomeServantImpl servant = 
                new org.objectweb.ccm.Containers.HomeServantImpl();

            //Remove but will fail when removing component
            //servant.the_container_base(this);
            home_exec._home_servant(servant);
            
            servant.the_home_executor(home_exec);

            //servant.the_home_configuration(get_home_configuration());
         
            //servant.the_component_configuration(get_component_configuration());
            servant.the_home_finder(home_manager_);

            // create and set the home reference.
            org.omg.Components.CCMHome ref = rootPCA_.activate_home(servant);

            // begin MI modif      
            /////////////////////////////////////////////    
            // create the home type MI for the given home    
            /////////////////////////////////////////////    
            //  retrieving IDL Declaration name         
            String IDL_name = retrieve_MI_name(entrypt);  

            if (IDL_name != null)           
            {
                //  setting the class and method name    
                String methodName = IDL_name + "_homeMI.create_type_MI";     
                // call the computed name method (static)     
                try
                {
                    home_type_ = (HomeType) TheURLClassLoader.getURLClassLoader()   
                        .callStaticClassMethod(methodName,new Object[0]);     
                }
                catch(Exception e)
                {
                    // no home type entypt is available
                    home_type_ = null;
                }
            }
        
            ///////////////////////////// MODIF MetaInformation   
            
            HomeInstance _tmp_hi = null;
            
            if (home_type_ != null){ 
                _tmp_hi = home_type_.create_home_instance();     
                if (_tmp_hi == null){ 
                    trace_error("install_home", "home instance is null ", null);   
                }            
                servant.the_home_instance(_tmp_hi);   
            } 
            else {
                trace_error("retrieve_MI_name",
                            "Instance MetaInformation for DCI not supported ", null);           
            }
            servant.the_home_instance(_tmp_hi);
            // End of MI modif
            
            //Remove but can fail 
            //listener_.on_create(ref);
            

            return ref;
        }
        catch(Exception e){
            e.printStackTrace();
            System.err.println(e.getMessage());
        }

        return null;

    }
    




}
