/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

// Package dependencies.
import org.objectweb.corba.ast.api.DeclarationKind;
import org.objectweb.corba.ast.api.AbstractStorageHomeDecl;
import org.objectweb.corba.ast.api.StringList;
import java.util.ArrayList;

/**
 * SegmentDeclImpl is a wrapper class for
 * CIDL segment declarations.
 *
 *
 * Inherits from:
 *
 * - DeclarationImpl as they are IDL declarations,
 *
 * - SegmentDecl: OMG IDL for SegmentDecl.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.3
 */

public class SegmentDeclImpl
     extends DeclarationImpl
  implements org.objectweb.corba.ast.api.SegmentDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The associated abstract storagehome. */
    private AbstractStorageHomeDeclImpl ash_;

    /** The list of facets. */
    private StringListImpl facets_;

    /** The list of sinks. */
    private StringListImpl sinks_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the declaration.
     */
    protected
    SegmentDeclImpl(Repository rep,
                    ScopeImpl parent)
    {
        // Call the DeclarationImpl constructor.
        super(rep, parent);

        // Init internal state.
        ash_ = null;
        facets_ = new StringListImpl();
        sinks_ = new StringListImpl();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Remove implemented facets for the list.
     */
    protected void
    computeFacetsList( org.objectweb.corba.ast.api.ComponentDecl comp,
                       ArrayList implemented_facets )
    {
        org.objectweb.corba.ast.api.InterfaceDecl itf = null;
        org.objectweb.corba.ast.api.ComponentDecl current = null;
        String[] facets = null;
        String[] sinks = null;

        /** the CCM_<providing_object_name> interface **/
          // For each facet implemented, add the corresponding interface
        facets = getFacetList().getStrings();
        for (int i=0; i<facets.length; i++)
        {
            org.objectweb.corba.ast.api.InterfacePortDecl itf_port = null;

            current = comp;
            while ((itf_port == null) && (current != null))
            {
                itf_port = (org.objectweb.corba.ast.api.InterfacePortDecl)current.lookup(facets[i]);
                current = current.getBaseComponent();
            }
            implemented_facets.remove(itf_port);
        }

          // For each sink implemented, add the corresponding interface
        sinks = getSinkList().getStrings();
        for (int i=0; i<sinks.length; i++)
        {
            org.objectweb.corba.ast.api.EventPortDecl event_port = null;

            current = comp;
            while ((event_port == null) && (current != null))
            {
                event_port = (org.objectweb.corba.ast.api.EventPortDecl)current.lookup(sinks[i]);
                current = current.getBaseComponent();
            }
            implemented_facets.remove(event_port);
        }
    }

    /**
     * Declare CIF interfaces into Interface Repository.
     */
    protected void
    declareCIF(org.objectweb.corba.ast.api.ModuleDecl module,
               org.objectweb.corba.ast.api.InterfaceDecl base,
               org.objectweb.corba.ast.api.ComponentDecl comp)
    {
        org.objectweb.corba.ast.api.InterfaceList list = null;
        org.objectweb.corba.ast.api.InterfaceDecl itf = null;
        String[] facets = null;
        String[] sinks = null;

        /** Create the segment interface : CIF_<segment_name> **/
        itf = module.declareLocalInterface("CIF_"+getName());
          // Feed inherited interface list
        list = itf.getInheritedInterfaceList();
          // the CIF_SegmentBase interface
        list.add(base);

        /** the CCM_<providing_object_name> interface **/
          // For each facet implemented, add the corresponding interface
        facets = getFacetList().getStrings();
        for (int i=0; i<facets.length; i++)
        {
            org.objectweb.corba.ast.api.InterfacePortDecl itf_port = null;
            org.objectweb.corba.ast.api.ComponentDecl current = null;

            current = comp;
            while ((current != null) && (itf_port == null))
            {
                itf_port = (org.objectweb.corba.ast.api.InterfacePortDecl)current.lookup(facets[i]);
                current = comp.getBaseComponent();
            }
            list.add( itf_port.getLocalMapping() );
        }

          // For each sink implemented, add the corresponding interface
        sinks = getSinkList().getStrings();
        for (int i=0; i<sinks.length; i++)
        {
            org.objectweb.corba.ast.api.EventPortDecl event_port = null;
            org.objectweb.corba.ast.api.ComponentDecl current = null;

            current = comp;
            while ((current != null) && (event_port == null))
            {
                event_port = (org.objectweb.corba.ast.api.EventPortDecl)current.lookup(sinks[i]);
                current = comp.getBaseComponent();
            }
            list.add( event_port.getEvent().getLocalConsumerMapping() );
        }

        itf.create();

        /** Create operations for persistence if needed **/
        if (getAbstractStorageHome() != null)
        {
            org.objectweb.corba.ast.api.NativeDecl ast = null, ash = null;
            org.objectweb.corba.ast.api.OperationDecl op = null;

            ash = getAbstractStorageHome().getMapping();
            ast = getAbstractStorageHome().getAbstractStorageType().getMapping();

            op = itf.startOperation("get_state");
            op.setType(ast);
            op.create();
            // Create an operation to get home ?
        }
    }

    // ==================================================================
    //
    //  Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the any declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
       return null;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * This method is implemented into DeclarationImpl subclasses.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
      return DeclarationKind.dk_segment;
    }

    /**
      * Obtain its category (IDL, PSDL, CIDL).
      *
      * @return The category of the declaration.
      */
    public org.objectweb.corba.ast.api.DeclarationCategory
    getCategory()
    {
        return org.objectweb.corba.ast.api.DeclarationCategory.dc_cidl;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.SegmentDecl
    //
    // ==================================================================

    /**
     * Set the associated abstract storagehome.
     *
     * @param ash The associated abstract storagehome.
     */
    public void
    setAbstractStorageHome(AbstractStorageHomeDecl ash)
    {
        ash_ = (AbstractStorageHomeDeclImpl)ash;
    }

    /**
     * Obtain the associated abstract storagehome.
     *
     * @return The associated abstract storagehome.
     */
    public AbstractStorageHomeDecl
    getAbstractStorageHome()
    {
        return ash_;
    }

    /**
     * Obtain the list of implemented facets.
     *
     * @return The list of facets.
     */
    public StringList
    getFacetList()
    {
        return facets_;
    }

    /**
     * Obtain the list of implemented sinks.
     *
     * @return The list of sinks.
     */
    public StringList
    getSinkList()
    {
        return sinks_;
    }
}
