/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey, Philippe Merle.
Contributor(s): .

====================================================================*/

package org.objectweb.corba.command.lib;

// Package dependencies.
import org.objectweb.corba.command.api.Compiler;
import org.objectweb.util.cmdline.api.CommandLine;
import org.objectweb.util.cpp.lib.PreprocessorApplication;

/**
 * Abstract base class for all compiler commands.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 1.2
 */

public abstract class CompilerBase
              extends CommandOnASTBase
           implements Compiler
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The PreprocessorApplication. */
    private PreprocessorApplication preprocessor_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the command line manager.
     *
     * @param commandLine The command line manager.
     */
    public
    CompilerBase(CommandLine commandLine)
    {
        this(commandLine, true);
    }

    /**
     * The constructor with the command line manager
     * and boolean to manage options.
     *
     * @param commandLine The command line manager.
     * @param withPreprocessorOptions True to add preprocessor options in the command line.
     */
    public
    CompilerBase(CommandLine commandLine, boolean withPreprocessorOptions)
    {
        // Calls the CommandOnASTBase constructor.
        super(commandLine);

        // Inits internal state.
        preprocessor_ = new PreprocessorApplication(commandLine, false, withPreprocessorOptions);
        preprocessor_.setConsole(getConsole());
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cmdline.api.Application
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.command.api.Application
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.command.api.CommandOnIR3
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.command.api.CommandOnAST
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.command.api.Compiler
    //
    // ==================================================================

    /**
     * Compiles the OMG IDL file and feeds the OpenCCM Interface Repository.
     *
     * @param filename The file name to compile.
     *
     * @return The associated AST FileScope if ok, or else null.
     */
    public org.objectweb.corba.ast.api.FileScope
    compile(String filename)
    {
        if (!new java.io.File(filename).exists())
            getConsole().error("File " + filename + " not found!");

        getConsole().message("Reading from file " + filename + "...");

        // Preprocesses the file.
        java.io.File file = preprocessor_.preprocess(filename);
        if(file == null) return null;

        // Creates a new parser.
        org.objectweb.corba.parser.lib.ErrorManager errorManager = null;
        org.objectweb.corba.parser.api.Parser parser = null;
        try
        {
            errorManager = new org.objectweb.corba.parser.lib.ErrorManager(filename);
            parser = new org.objectweb.corba.parser.lib.Parser(
                             new java.io.FileInputStream(file),
                             getAST(),
                             errorManager);
        }
        catch(java.io.FileNotFoundException e)
        {
            getConsole().error("File " + file.toString() + " not found!");
            return null;
        }

        // Parses the file.
        getConsole().message("Compiling " + filename + " file...");
        org.objectweb.corba.ast.api.FileScope result = parser.parseCIDL();

        int nbErrors = errorManager.getNbErrors();
        int nbWarnings =  errorManager.getNbWarnings();

        if (result == null)
        {
            String msg = "Compilation failed: " + nbErrors + " error";
            if(nbErrors > 1) msg += 's';
            msg += ", " + nbWarnings + " warning";
            if(nbWarnings > 1) msg += 's';
            msg += '.';
            getConsole().error(msg);
            return null;
        }

        String msg = "Compilation completed: " + nbWarnings + " warning";
        if(nbWarnings > 1) msg += 's';
        msg += '.';
        getConsole().message(msg);

        // Deletes the temporary file generated by the preprocessor.
        try
        {
            file.delete();
        }
        catch(Exception e)
        {
            getConsole().error("Problem when deleting temporary file " + file.toString());
            report_exception(e);
        }

        return result;
    }

    /**
     * Gets the preprocessor of the compiler.
     *
     * @return The preprocessor.
     */
    public org.objectweb.util.cpp.api.PreprocessorApplication
    getPreprocessorApplication()
    {
        return preprocessor_;
    }    

    // ==================================================================
    //
    // Other public methods.
    //
    // ==================================================================
}
