/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey, Philippe Merle.
Contributor(s): .

====================================================================*/

package org.objectweb.corba.command.lib;

// Package dependencies.
// import org.objectweb.corba.command.api.IR3toIDL2;
import org.objectweb.util.cmdline.lib.DefaultCommandLine;
import java.util.ArrayList;

/**
 * Implementation of the ir3_idl2 command.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.2
 */

public class IR3toIDL2
     extends GeneratorBase
  implements org.objectweb.corba.command.api.IR3toIDL2
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Include Options. */
    private OptionMultipleArguments include_;
    private OptionMultipleArguments userInclude_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    IR3toIDL2()
    {
        // Calls the GeneratorBase constructor.
        super(new DefaultCommandLine("ir3_idl2",
                                     "declaration",
                                     "Generate OMG IDL2 for a declaration from the OpenCCM Interface Repository",
                                     true),
              "generated.idl");
 
        // Inits internal state.
        // Adds the include option.
        include_ = new OptionMultipleArguments(
                           "-i",
                           new String[] { "-i" },
                          "file",
                          new String[] { "Add a #include <file> statement" },
                          "",
                          true );
        getCommandLine().addOption(include_);

        // Adds the user include option.
        userInclude_ = new OptionMultipleArguments(
                           "-ipath",
                           new String[] { "-ipath" },
                           "file",
                           new String[] {
                               "Add a #include \"file\" statement where",
                               "file must be the complete file path"
                           },
                           "",
                           true);
        getCommandLine().addOption(userInclude_);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cmdline.api.Application
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.command.api.Application
    //
    // ==================================================================

    /**
     * Runs the application.
     *
     * @param args The command line arguments.
     *
     * @return The status.
     */
    public int
    run(java.lang.String[] args)
    {
        // Inits the OpenCCM Abstract Syntax Tree.
        if(!initAST())
            return -1;

        // Computes the _local file name.
        String localfilename = getOutputFileName();
        int index = localfilename.lastIndexOf('.');
        if ((index == -1) || (index == localfilename.length()))
            localfilename += "_local";
        else
            localfilename = localfilename.substring(0, index) + "_local"
                          + localfilename.substring(index);

        // Generates OMG IDL2.
        if (!generate_idl2(args[0],
                           getOutputFileName(),
                           localfilename,
                           include_.getOptionArguments(),
                           userInclude_.getOptionArguments()))
            return -1;

        // All is OK.
        return 0;
    }

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.command.api.CommandOnIR3
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.command.api.CommandOnAST
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.command.api.Generator
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for org.objectweb.corba.command.api.IR3toIDL2
    //
    // ==================================================================

    /**
     * Generates OMG IDL2 for a scope.
     *
     * @param decl_name The scope to be generated.
     * @param outputfile The file name to write in.
     * @param outputfile_local The file name to write local declarations in.
     * @param includes Files to include with <...>.
     * @param userIncludes User files to include "...".
     *
     * @return True if ok, else false.
     */
    public boolean
    generate_idl2(String decl_name,
                  String outputfile,
                  String outputfile_local,
                  String[] includes,
                  String[] userIncludes)
    {
        // Switch the OpenCCM Interface Repository in mode OMG IDL 2.x.
        getComponentRepository().as_IDL2_repository();

        // Retrieves the scope to generate.
        org.objectweb.corba.ast.api.Scope scope = null;
        try
        {
            scope = (org.objectweb.corba.ast.api.Scope)getAST().lookup(decl_name);
        }
        catch(ClassCastException e)
        {
            getConsole().error(decl_name + "is not a scope!");
            return false;
        }

        if(scope == null)
        {
            getConsole().error("Declaration " + decl_name + " not found!");
            return false;
        }

        getConsole().message("Generating OMG IDL2 for " + decl_name + "...");

        // Adds include statements.
        ArrayList includes_list = new ArrayList( java.util.Arrays.asList(includes) );
        includes_list.add("Components.idl");

        // Launchs the generator.
        org.objectweb.corba.generator.idl.api.IDL2Generator gen =
                         new org.objectweb.corba.generator.idl.lib.IDL2Generator(getAST());
        gen.initialize( outputfile,
                        outputfile_local,
                        includes_list,
                        new ArrayList(java.util.Arrays.asList(userIncludes)),
                        scope,
                        "IR3toIDL2" );
        // Generates.
        try
        {
            gen.generate(scope);
        }
        catch(org.objectweb.corba.generator.common.lib.GenerationException ex)
        {
            report_exception(ex);
            return false;
        }

        // Gets an idl3 repository
        getComponentRepository().as_IDL3_repository();

        if (outputfile.equals(outputfile_local))
            getConsole().message("OMG IDL2 generated in " + outputfile + ".");
        else
            getConsole().message("OMG IDL2 generated in " + outputfile + " and " + outputfile_local + ".");
        return true;
    }

    // ==================================================================
    //
    // Static public methods.
    //
    // ==================================================================

    /**
     * The main bootstrap method.
     *
     * @param args The command line arguments.
     */
    public static void
    main(String[] args)
    {
        IR3toIDL2 ir3_idl2 = new IR3toIDL2();
        ir3_idl2.runMain(args);
    }
}
