/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.gui.core;

/** The console's imports */
import org.objectweb.util.browser.naming.api.Entry;
import org.objectweb.util.browser.naming.api.Context;
import org.objectweb.util.browser.naming.api.ContextContainer;
import org.objectweb.util.browser.naming.api.Wrapper;
import org.objectweb.util.browser.naming.api.EntryIterator;
import org.objectweb.util.browser.naming.api.ExtendedEntry;
import org.objectweb.util.browser.naming.api.ContextContainerFactory;
import org.objectweb.util.browser.naming.wrappers.common.DefaultEntry;
import org.objectweb.util.browser.naming.wrappers.common.DefaultName;
import org.objectweb.util.browser.naming.wrappers.common.DefaultContextContainerFactory;
import org.objectweb.util.browser.api.BrowserProperty;
import org.objectweb.util.browser.api.ContextProperty;
import org.objectweb.util.browser.api.TreeConfiguration;
import org.objectweb.util.browser.api.ViewPanel;
import org.objectweb.util.browser.gui.panel.WhitePanel;
import org.objectweb.util.browser.gui.panel.NodeViewPanel;
import org.objectweb.util.browser.xmlreader.decoder.ContextXMLDecoder;
import org.objectweb.util.browser.xmlreader.decoder.XMLDecoder;
import org.objectweb.util.browser.xmlreader.decoder.BrowserXMLDecoder;

/** The java API's imports */
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.io.File;

/** The java swing API's imports */
import javax.swing.JTree;
import javax.swing.JPopupMenu;
import javax.swing.JPanel;
import javax.swing.Icon;
import javax.swing.event.TreeWillExpandListener;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeExpansionListener;
import java.awt.Point;
import java.awt.Component;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

/** The java swing tree API's imports */
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.DefaultTreeSelectionModel;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

/**
 * Implementation of a Entry tree which is dynamic. 
 * You can customize icons, panels and popup menus 
 * for every objects you display.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class DynamicTree extends JTree implements TreeConfiguration {

    // ==================================================================
    //
    // Internal states.
    //
    // ==================================================================

    /** To access to the customization */
    protected AdminCustomization custom;

    /** Parent panel, use to refresh right panel */
    protected ViewPanel viewPanel_;

    /** Root node */
    protected DefaultMutableTreeNode rootNode;

    /** The initial context */
    protected ContextContainer initialContext_;

    /** Model to manipulate node */
    protected DefaultTreeModel treeModel;

    /** Model to define what to do when a node is selected */
    protected DefaultTreeSelectionModel selectionModel;

    /** Model to display node */
    protected DefaultTreeCellRenderer treeCellRenderer;

    /** A map which informs about the node state (expanded or collapsed) */
    protected Map nodeState_;

    /** To know the selected node */
    protected String selectedNode_;

    /** To know the selected treepath*/
    protected TreePath selectedTreePath_;

    /** Panel display when any panel was found */
    protected NodeViewPanel blankPanel_;

    /** The associated contextProperty */
    protected ContextProperty contextProperty_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Builds a new DynamicTree
     * @param initialContext The context to be displayed
     * @param initAll Uses to know if the constructor may initialize all
     */
    private void build(ContextContainer initialContext, boolean initAll) {

        initialContext_ = initialContext;

        rootNode = new DefaultMutableTreeNode(new DefaultEntry(initialContext_,new DefaultName("Root"),null));
        treeModel = new MyTreeModel(rootNode);
        setModel(treeModel);

        if (initAll) {
            nodeState_ = new HashMap();
            selectedNode_ = "";
            blankPanel_ = new WhitePanel();
            selectionModel = new DefaultTreeSelectionModel();
            selectionModel.setSelectionMode(
                TreeSelectionModel.SINGLE_TREE_SELECTION);
            setSelectionModel(selectionModel);
            addMouseListener(new MyMouseAdapter());
            addTreeWillExpandListener(new MyTreeWillExpandListener());
            addTreeExpansionListener(new MyTreeExpansionListener());
            addTreeSelectionListener(new MyTreeSelectionListener());
            setCellRenderer(new MyTreeCellRenderer());
        }

        setShowsRootHandles(true);
        collapseRow(0);
        expandRow(0);

        setRootVisible(false);
    }

    /**
     * Constructs a new DynamicTree.
     */
    public DynamicTree() {
        ContextContainerFactory ccf = new DefaultContextContainerFactory(null);
        build(ccf.create(), true);
    }

    /**
     * Constructs a new DynamicTree initialized with the specified context.
     * @param initialContext The context to be displayed
     */
    public DynamicTree(ContextContainer initialContext) {
        build(initialContext, true);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Adds an entry to a node.
     * @param parent Node which is the parent of the new entry
     * @param child Entry to add
     * @return The created node
     */
    protected DefaultMutableTreeNode addObject(
        DefaultMutableTreeNode parent,
        ExtendedEntry child) {
        if (custom != null) {
            Context context = custom.getContext(child.getOWValue());
            if (context != null) {
                child.setOWValue(context);
                child.setWrappedObject(((Wrapper) context).getWrapped());
            }
        }
        DefaultMutableTreeNode childNode = new DefaultMutableTreeNode(child);
        if (parent == null) {
            parent = rootNode;
        }
        treeModel.insertNodeInto(childNode, parent, parent.getChildCount());

        return childNode;
    }

    /**
     * Gets a string representation of the path
     * 
     * @param node The node
     * @return The path from th root to the node
     * 
     */
    protected String pathToString(DefaultMutableTreeNode node) {
        StringBuffer sf = new StringBuffer();
        Object[] o = node.getUserObjectPath();
        for (int i = 0; i < o.length; i++) {
            sf.append(((Entry) o[i]).getOWName().getOWId());
            if (i != o.length - 1)
                sf.append("/");
        }
        return sf.toString();
    }

    /**
     * Returns the entry associates to a node
     * @param o The node
     * @return The associated entry
     */
    protected Entry objectToEntry(Object o) {
        if (o != null) {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode) o;
            return (Entry) node.getUserObject();
        }
        return null;
    }

    /**
     * Shows the associated PopupMenu, if exists
     * @param p Point which indicates where is the mouse
     */
    protected void showMenu(Point p) {
        if (p != null) {
            TreePath path = getPathForLocation((int) p.getX(), (int) p.getY());
            if (path != null) {
                selectionModel.setSelectionPath(path);
                Entry entry = objectToEntry(getLastSelectedPathComponent());
                if (custom != null) {
                    JPopupMenu menu = custom.getMenu((ExtendedEntry) entry);
                    if (menu != null) {
                        menu.show((Component) this,(int) (p.getX()),(int) (p.getY()));
                    }
                }
            }
        }
    }

    /**
     * Gives the TreePath correspoding to the given id
     * @param id The id of the node of the tree
     * @return The associated TreePath or null
     */
    protected TreePath getPathFromInitialContext(String id) {
        if (id != null) {
            Enumeration enum = rootNode.children();
            while (enum.hasMoreElements()) {
                DefaultMutableTreeNode childNode = (DefaultMutableTreeNode) enum.nextElement();
                Entry entry = (Entry) childNode.getUserObject();
                if (entry.getOWName().getOWId().equals(id))
                    return new TreePath(childNode.getPath());
            }
        }
        return null;
    }

    /**
     * Open the tree nodes until a specific level
     *
     * @param root The root TreePath to develop
     * @param level The level (>0 for a specific level)
     */
    protected void openTree(TreePath root, int level) {
        if (level > 0) {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode) root.getLastPathComponent();
            Enumeration enum = node.children();
            while (enum.hasMoreElements()) {
                DefaultMutableTreeNode childNode = (DefaultMutableTreeNode) enum.nextElement();
                TreePath p = new TreePath(childNode.getPath());
                expandPath(p);
                openTree(p, level - 1);
            }
            //  	}else if (level <0){
            //    	    for(int i=0;i<getRowCount();i++)
            //    		expandRow(i);
        }
    }

    /**
     * Returns true if the context has at least one child. 
     * @param context The context to check
     * @return true if the context has children
     */
    protected boolean hasChild(Context context) {
        EntryIterator it = context.getOWEntries();
        return it.hasNext();
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /** 
     * Set a new property context.
     * @param files An array of XML files to parse and to apply (It can be a single file or a directory)
     */
    public void setNewBrowserProperty(File[] files) {
        if (files != null) {
            XMLDecoder config = new BrowserXMLDecoder();
            config.setPropertyFile(files);
            setNewBrowserProperty((BrowserProperty) config);
        }
    }

    /** 
     * Set a new property context
     */
    public void setNewBrowserProperty(BrowserProperty properties) {
        if (properties != null) {
            properties.setContextProperty(contextProperty_);
            custom = new AdminCustomization(properties, this);
        }
    }

    /**
     * Gives the current BrowserProperty. 
     * @return The current BrowserProperty
     */
    public BrowserProperty getBrowserProperty() {
        return custom.getBrowserProperty();
    }

    /**
     * Creates a new context property with the given file
     * @param file The properties file to parse and to apply
     */
    public void setNewContextProperty(File file) {
        if (file != null && file.exists()) {
            XMLDecoder config = new ContextXMLDecoder();
            config.setPropertyFile(file);
            setNewContextProperty((ContextProperty) config);
        }
    }

    /**
     * Sets a new context property
     * @param properties The properties to apply 
     */
    public void setNewContextProperty(ContextProperty properties) {
        if (properties != null) {
            contextProperty_ = properties;
            // Updates the initialContext_
            ContextContainerFactory ccf =
                new DefaultContextContainerFactory(properties.getDecoder());
            ContextContainer newContext = ccf.create();
            if (initialContext_ != null) {
                EntryIterator it = initialContext_.getOWEntries();
                while (it.hasNext()) {
                    Entry entry = it.next();
                    newContext.addEntry(entry.getOWName().getOWId(),entry.getOWValue());
                }
            }
            build(newContext, false);
        }
    }

    /**
     * Gives the current ContextProperty.
     * @return The current ContextProperty
     */
    public ContextProperty getContextProperty() {
        return contextProperty_;
    }

    /**
     * Returns the selected entry
     */
    public Entry getSelectedEntry() {
        Object object = getLastSelectedPathComponent();
        if (object != null)
            return objectToEntry(object);
        return null;
    }

    /**
     * Returns the parent of the selected entry
     */
    public Entry getSelectedEntryParent() {
        Object object = getLastSelectedPathComponent();
        if (object != null)
            return objectToEntry(((DefaultMutableTreeNode) object).getParent());
        return null;
    }

    /**
     * Refresh all the tree
     */
    public void refreshAll() {
        TreePath p = new TreePath(rootNode.getPath());
        collapsePath(p);
        expandPath(p);
    }

    /**
     * Refresh the selected node
     */
    public void refreshSelectedNode() {
        TreePath path = getSelectionPath();
        DefaultMutableTreeNode node = (DefaultMutableTreeNode) path.getLastPathComponent();
        if (isExpanded(path)) {
            collapsePath(path);
            expandPath(path);
        } else if (node.isLeaf()) {
            fireTreeExpanded(path);
        }
    }

    /**
     * Add an object into the initial context
     *  
     * @param id Name of the object
     * @param object The abject to add
     */
    public void addToInitialContext(String id, Object object) {
        addToInitialContext(id, object, 0);
    }

    /**
     * Add an object into the initial context and expands the ndoe until a specific level if this one is a context
     *  
     * @param id Name of the object
     * @param object The object to add
     * @param level The level (>0 for a specific level)
     */
    public void addToInitialContext(String id, Object object, int level) {
        if (id != null && object != null) {
            initialContext_.addEntry(id, object);
            refreshAll();
            if (level > 0) {
                TreePath path = getPathFromInitialContext(id);
                if (path != null) {
                    expandPath(path);
                    openTree(path, level - 1);
                }
            }
        }
    }

    /**
     * Gives the initial context 
     * @return The initial context
     */
    public Context getInitialContext() {
        return initialContext_;
    }

    /**
     * Returns true if this initial context contains an entry for the specified id.
     * @param id Key whose presence in this initial context is to be tested.
     * @return True if this initial context contains an entry for the specified id.
     */
    public boolean containsEntry(String id) {
        Entry entry = initialContext_.getOWLocalEntry(id);
        return entry != null;
    }

    /**
     * Fixes the view panel for the tree (Where the associated panels will be displayed).
     * @param viewPanel The panel to use to display associated panels
     */
    public void setViewPanel(ViewPanel viewPanel) {
        viewPanel_ = viewPanel;
    }

    // ==================================================================
    //
    // Internal classes.
    //
    // ==================================================================

    /**
     * Mouse Adapter
     * Use to draw Popup Menu
     */
    final class MyMouseAdapter extends MouseAdapter {
        public void mousePressed(MouseEvent e) {
            popupLayout(e);
        }
        public void mouseReleased(MouseEvent e) {
            popupLayout(e);
        }
        private void popupLayout(MouseEvent e) {
            if (e.isPopupTrigger()) {
                showMenu(e.getPoint());
            }
        }
    }

    /**
     * Tree Will Expand Listener
     * Use to complete node : it adds child to current node
     */
    final class MyTreeWillExpandListener implements TreeWillExpandListener {
        public void treeWillCollapse(TreeExpansionEvent event) {
            DefaultMutableTreeNode currentNode = (DefaultMutableTreeNode) event.getPath().getLastPathComponent();
            Entry e = (Entry) currentNode.getUserObject();
            nodeState_.put(pathToString(currentNode), new GraphicState(false));
        }

        public void treeWillExpand(TreeExpansionEvent event) {
            DefaultMutableTreeNode currentNode = (DefaultMutableTreeNode) event.getPath().getLastPathComponent();
            Entry e = (Entry) currentNode.getUserObject();
            Object userObject = e.getOWValue();
            nodeState_.put(pathToString(currentNode), new GraphicState(true));
            currentNode.removeAllChildren();
            EntryIterator it = ((Context) userObject).getOWEntries();
            while (it.hasNext()) {
                Entry entry = it.next();
                DefaultMutableTreeNode childNode = addObject(currentNode, (ExtendedEntry) entry);
            }
            treeModel.reload(currentNode);
        }
    }

    /**
     * Tree Expansion Listener
     * Use to refresh node
     */
    final class MyTreeExpansionListener implements TreeExpansionListener {
        public void treeCollapsed(TreeExpansionEvent event) {
        }

        public void treeExpanded(TreeExpansionEvent event) {
            DefaultMutableTreeNode node =
                (DefaultMutableTreeNode) (event.getPath()).getLastPathComponent();
            if (!node.isLeaf()) {
                Enumeration enum = node.children();
                while (enum.hasMoreElements()) {
                    DefaultMutableTreeNode childNode = (DefaultMutableTreeNode) enum.nextElement();
                    Entry entry = (Entry) childNode.getUserObject();
                    if (nodeState_.containsKey(pathToString(childNode))) {
                        GraphicState gs = (GraphicState) nodeState_.get(pathToString(childNode));
                        if (gs.isExpanded()) {
                            expandPath(new TreePath(childNode.getPath()));
                        }
                    }

                    if (selectedNode_.equals(pathToString(childNode))) {
                        setSelectionPath(
                            new TreePath(treeModel.getPathToRoot(childNode)));
                    }
                }
            }
        }
    }

    /** 
     * Tree Selection Listener 
     * Use to put the panel corresponding to the selected node
     */
    final class MyTreeSelectionListener implements TreeSelectionListener {

        public void valueChanged(TreeSelectionEvent e) {
            validate();
            Entry entry = null;
            Object object = getLastSelectedPathComponent();
            String newSelectedNode = null;
            if (object != null) {
                DefaultMutableTreeNode node = (DefaultMutableTreeNode) object;
                if (!node.equals(rootNode)) {
                    newSelectedNode = pathToString(node);
                }
                entry = (Entry) node.getUserObject();
            }
            if (selectedNode_ != null && newSelectedNode != null && !selectedNode_.equals(newSelectedNode)) {
                selectedNode_ = newSelectedNode;
                NodeViewPanel panel = null;
                if (entry != null && custom != null) {
                    panel = custom.getPanel((ExtendedEntry) entry);
                    if (panel != null && viewPanel_ != null) {
                        viewPanel_.setPanel(panel);
                    }
                }
                if (entry == null || custom == null || panel == null) {
                    if (viewPanel_ != null)
                        viewPanel_.setPanel(blankPanel_);
                }
            }
        }
    }

    /**
     * Tree Cell Renderer
     * Use to put the icon corresponding to the class 
     * of the Value object of the Entry
     */
    final class MyTreeCellRenderer extends DefaultTreeCellRenderer {

        public Component getTreeCellRendererComponent(JTree tree,Object value,boolean selected,boolean expanded,boolean leaf,int row,boolean hasFocus) {
        	super.getTreeCellRendererComponent(tree,value,selected,expanded,leaf,row,hasFocus);
            //  	    if(selected){
            //  		setBackground(getTextSelectionColor());
            //  	    }
            // Put the appropriate icon
            Entry entry = objectToEntry(value);
            if (custom != null) {
                Icon icon = custom.getIcon((ExtendedEntry) entry);
                if (icon != null)
                    setIcon(icon);
            }
            // Put the name of the entry as the label of the node
            setText(entry.getOWName().getOWId());
            return this;
        }

    }

    /**
     * Tree Model
     * Uses to know if a node is leaf or not
     */
    final class MyTreeModel extends DefaultTreeModel {

        public MyTreeModel(TreeNode root) {
            super(root);
        }

        public boolean isLeaf(Object node) {
            Entry entry = objectToEntry(node);
            // An entry is a leaf if it is not a Context instance or if it's a Context instance without child.
            Object o = entry.getOWValue();
            return (!(o instanceof Context) || ((o instanceof Context) && !hasChild((Context) o)));
        }

    }

    /**
     * Uses to know if a node is expanded
     */
    protected class GraphicState {

        protected boolean isExpanded_;

        public GraphicState(boolean isExpanded) {
            isExpanded_ = isExpanded;
        }

        public boolean isExpanded() {
            return isExpanded_;
        }

        public void isEspanded(boolean value) {
            isExpanded_ = value;
        }

    }

}
