/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.plugins.java.naming;

/** To use the EntryIterator interface. */
import org.objectweb.util.browser.naming.api.EntryIterator;

/** To use the DefaultName class. */
import org.objectweb.util.browser.naming.wrappers.common.DefaultName;

/** To use the DefaultEntry class. */
import org.objectweb.util.browser.naming.wrappers.common.DefaultEntry;

/** To use the Entry interface. */
import org.objectweb.util.browser.naming.api.Entry;

/**
 * Basic implementation of the EntryIterator interface
 * on top of a java.util.Properties instance.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * @version 0.1
 */

public class JavaPropertiesIterator implements EntryIterator {
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The iterator of the keys. */
    private java.util.Enumeration keys_;

    /** The iterator of the elements. */
    private java.util.Enumeration elements_;

    /** The owner Context. */
    private JavaPropertiesContext context_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Constructs a DefaultEntryIterator from a Java Properties instance.
     *
     * @param properties The Java properties.
     * @param context The owner context.
     */
    public JavaPropertiesIterator(
        java.util.Properties properties,
        JavaPropertiesContext context) {
        // Inits internal state.
        keys_ = properties.keys();
        elements_ = properties.elements();
        context_ = context;
    }

    // ==================================================================
    //
    // No internal methods.
    //
    // ==================================================================

    /**
     * Creates an Entry.
     *
     * @param value The value of the entry to create.
     * @param id The id of the name of the entry to create.
     *
     * @return A new Entry.
     *
     * @postcondition
     *              return.getOWValue() == value
     * and return.getOWName().getOWID() == id
     * and        return.getOWContext() == this;
     */
    protected Entry createEntry(Object value, String id) {
        return new DefaultEntry(value, new DefaultName(id), context_);
    }

    // ==================================================================
    //
    // Public methods for interface EntryIterator.
    //
    // ==================================================================

    /**
     * Returns true if the iteration has more entries.
     *
     * In other words, returns true if next would return an entry
     * rather than throwing an exception.
     *
     * @return true if the iterator has more entries.
     */
    public boolean hasNext() {
        return keys_.hasMoreElements();
    }

    /**
     * Returns the next entry in the interation.
     *
     * @return The next entry in the iteration.
     *
     * @exception NoSuchElementException Iteration has no more entries.
     */
    public Entry next() {
        return createEntry(
            elements_.nextElement(),
            (String) keys_.nextElement());
    }

    /**
     * Removes from the underlying collection the last entry returned by
     * the iterator (optional operation). This method can be called only
     * once per call to next. The behavior of an iterator is unspecified
     * if the underlying collection is modified while the iteration is
     * in progress in any way other than by calling this method.
     *
     * @exception UnsupportedOperationException
     *            If the remove operation is not supported by
     *            this EntryIterator.
     *
     * @exception IllegalStateException
     *            If the next method has not yet been called,
     *            or the remove method has already been called
     *            after the last call to the next method.
     */
    public void remove() {
        // TODO: Must be implemented!!!!
        throw new UnsupportedOperationException();
    }
}
