/*====================================================================

ObjectWeb Util CommandLine Package.
Copyright (C) 2003 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Christophe Contreras.

====================================================================*/

package org.objectweb.util.cmdline.lib;

// Package dependencies.
import org.objectweb.util.api.ExceptionWrapper;
import org.objectweb.util.cmdline.api.Application;
import org.objectweb.util.cmdline.api.CommandLine;
import org.objectweb.util.cmdline.api.Console;

/**
 * Abstract base class for Application instances.
 *
 * This provides the following properties:
 *
 * - MailingList: The associated mailing list of this development.
 *
 * - VersionInformation: The associated version information.
 *
 * This provides the following methods:
 *
 * - main and start: Starts the application.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

abstract public class ApplicationBase
              extends DefaultCommandLineHolder
           implements org.objectweb.util.cmdline.api.Application
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The associated console. */
    private Console console_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /** The default constructor. */
    public
    ApplicationBase()
    {
        this(null, false, true);
    }

    /**
     * The constructor with the command line manager.
     *
     * @param commandLine The command line manager.
     */
    public
    ApplicationBase(CommandLine commandLine)
    {
        this(commandLine, false, true);
    }

    /**
     * The constructor with the command line manager.
     *
     * @param commandLine The command line manager.
     * @param withDefaultOptions True to add default options in the command line.
     */
    public
    ApplicationBase(CommandLine commandLine,
                    boolean withDefaultOptions)
    {
        this(commandLine, false, withDefaultOptions);
    }

    /**
     * The constructor with the command line manager.
     *
     * @param commandLine The command line manager.
     * @param silentMessage Boolean for indicating if messages should be displayed.
     * @param withDefaultOptions True to add default options on the command line.
     */
    public
    ApplicationBase(CommandLine commandLine,
                    boolean silentMessage,
                    boolean withDefaultOptions)
    {
        // Calls the DefaultCommandLineHolder constructor.
        super(commandLine);

        // Inits internal state.
        setConsole(new DefaultConsole(getIdentity(), silentMessage));

        if(withDefaultOptions)
        {
            // Adds the --version command line option.
            if(withDefaultOptions && commandLine != null)
                commandLine.addOption(new DefaultOptionVersion(this));

            // Adds the --silent command line option.
            if(commandLine != null)
                commandLine.addOption(new DefaultOptionSilentMessage(console_));
        }
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Reports a Java system property.
     *
     * @param propertyName The name of the property to report.
     */
    protected void
    report_property(String propertyName)
    {
       getConsole().getErrorStream().
       println("System property " + propertyName + " = " +
               System.getProperty(propertyName));
    }

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.api.Identifiable
    //
    // ==================================================================

    /**
     * Obtains its identity.
     *
     * Should be defined in subclasses.
     *
     * @return Its identity.
     */
    public String
    getIdentity()
    {
        return this.getClass().getName();
    }

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cmdline.api.CommandLineHolder
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cmdline.api.ConsoleHolder
    //
    // ==================================================================

    /**
     * Obtains the associated console.
     *
     * @return The associated console.
     */
    public Console
    getConsole()
    {
        return console_;
    }

    /**
     * Sets the associated console.
     *
     * @param console The associated console.
     */
    public void
    setConsole(Console console)
    {
        console_ = console;
        getCommandLine().setConsole(console);
    }

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cmdline.api.Application
    //
    // ==================================================================

    /**
     * Obtains the mailing list associated to this development.
     *
     * @return The mailing list associated to this development.
     */
    public String
    getMailingList()
    {
        return "openccm@objectweb.org";
    }

    /**
     * Obtains the associated version information.
     *
     * @return The associated version information.
     */
    public String[]
    getVersionInformation()
    {
        return new String [] {
            getIdentity(),
            "",
            "Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL",
            "This is free software; you can redistribute it and/or",
            "modify it under the terms of the GNU Lesser General Public",
            "License as published by the Free Software Foundation; either",
            "version 2.1 of the License, or any later version.",
            "",
            "This is distributed in the hope that it will be useful,",
            "but WITHOUT ANY WARRANTY; without even the implied warranty of",
            "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU",
            "Lesser General Public License for more details.",
            "",
            "You should have received a copy of the GNU Lesser General Public",
            "License along with this library; if not, write to the Free Software",
            "Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA"
        };
    }

    /**
     * Reports an exception stack.
     *
     * @param exception The exception to report.
     */
    public void
    report_exception(Exception exception)
    {
       java.io.PrintStream err = getConsole().getErrorStream();
       err.print("Please report the following exception trace to ");
       err.println(getMailingList());
       err.println("including your used ORB name and version.");
       report_property("os.name");
       report_property("os.version");
       report_property("java.version");
       exception.printStackTrace();
    }

    /**
     * Executes the main function, i.e.
     * 1 - arguments = getCommandLine().parse(args);
     * 2 - status = starts(arguments);
     * 3 - System.exit(status);
     *
     * @param args The command line arguments.
     */
    public void
    runMain(String[] args)
    {
        try
        {
            // Parses the command line arguments.
            String[] arguments = getCommandLine().parse(args);

            // Starts the application.
            int status = start(arguments);

            // Exits according to the status of the application.
            System.exit(status);
        }
        // If any exception thrown then reports it.
        //
	catch(ExceptionWrapper exc)
	{
	    report_exception(exc.getException());
	}
	catch(Exception exc)
	{
	    report_exception(exc);
	}
    }

    /**
     * Starts the main function.
     *
     * Must be defined into subclasses.
     *
     * @param args The command line arguments.
     *
     * @return The status of the application.
     */
    abstract public int
    start(String[] args);
}
