/*===========================================================================

ObjectWeb Util Launcher package.
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ___________________________________________.

===========================================================================*/

package org.objectweb.util.launcher;


/** To use the Iterator class */
import java.util.Iterator;

/** To use generated classes */
import org.objectweb.util.launcher.launcherConfig.*;

/**
 * Parses a Java object wihch representes an XML Configuration.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 * @version 0.1
 */
public class XMLDecoderImpl
  implements XMLDecoder
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** A table with identifiants and their binding object */
    private java.util.HashMap objects_;

    /** The class name */
    private String className_;

    /** The urls list */
    private java.util.ArrayList urls_;

    /** The arguments list */
    private java.util.ArrayList arguments_;

    /** The properties list */
    private java.util.Properties properties_;

    /** The already parsed file list */
    private java.util.ArrayList parsed_files_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Default constructor
     */
    public XMLDecoderImpl()
    {
        objects_ = new java.util.HashMap();
        className_ = null;
        urls_ = new java.util.ArrayList();
        arguments_ = new java.util.ArrayList();
        properties_ = new java.util.Properties();
        parsed_files_ = new java.util.ArrayList();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Replace tag @NAME@ in a string.
     * NAME must be a jave property set.
     *
     * @param s - The string to replace.
     *
     * @return A new String with all tag replacements.
     */
    private String
    replaceTag(String s)
    {
        int first = s.indexOf("@");

        if (first != -1)
        {
            int next  = s.indexOf("@", first+1);

            if (next != -1)
            {
                String tag_value = "",
                       first_part = "",
                       last_part = "";

                try{
                    tag_value = System.getProperty( s.substring(first+1, next) );
                }catch(IndexOutOfBoundsException ex){
                    // Nothing to do
                }
                try{
                     first_part = s.substring(0,first);
                }catch(IndexOutOfBoundsException ex){
                    // Nothing to do
                }
                try{
                     last_part = s.substring(next+1);
                }catch(IndexOutOfBoundsException ex){
                    // Nothing to do
                }
                return replaceTag(first_part + tag_value + last_part);
            }
        }
        return s;
    }

    /**
     * Parses a configuration.
     *
     * @param conf - The Configuration to parse.
     */
    private void
    parseConfiguration(LauncherConfig config)
    {
        if (config != null)
        {
            java.util.List elements = null;

            // Import xml files
            elements = config.getIncludeList();
            for(Iterator i = elements.iterator() ; i.hasNext() ; )
            {
                Include incl = (Include)i.next();

                if ( !parsed_files_.contains(incl.getUrl()) )
                {
                    parsed_files_.add(incl.getUrl());
                    parseFile(incl.getUrl());
                }
/*                else
                {
                    System.err.println("Loop detected on "+incl.getUrl());
                    System.exit(-1);
                } */
            }

            // Add classpath tags
            elements = config.getClasspathList();
            for(Iterator i = elements.iterator() ; i.hasNext() ; )
            {
                Classpath c = (Classpath)i.next();

                objects_.put(c.getId(), c);
            }

            // Add arguments tags
            elements = config.getArgumentsList();
            for(Iterator i = elements.iterator() ; i.hasNext() ; )
            {
                Arguments a = (Arguments)i.next();

                objects_.put(a.getId(), a);
            }

            // Add properties tags
            elements = config.getPropertiesList();
            for(Iterator i = elements.iterator() ; i.hasNext() ; )
            {
                Properties p = (Properties)i.next();

                objects_.put(p.getId(), p);
            }

            // Add run tags
            elements = config.getRunList();
            for(Iterator i = elements.iterator() ; i.hasNext() ; )
            {
                Run r = (Run)i.next();

                objects_.put(r.getId(), r);
            }

        }
    }

    /**
     * Parses a Classpath element.
     *
     * @param id - The Classpath id to parse.
     */
    private void
    parseClasspath(String id)
    {
        java.util.List elements = null;
        Path path = null;
        Classpath cp = null;

        try
        {
            cp = (Classpath)objects_.get(id);
        }
        catch (ClassCastException e1)
        {
            System.err.println("'" + id + "' is not a Classpath element.");
        }

        if (cp != null)
        {
            elements = cp.getPathList();
            for(Iterator i = elements.iterator() ; i.hasNext() ; )
            {
                path = (Path)i.next();
                if (path.getUrl() != null)
                {
                    urls_.add( path.getUrl() );
                }
                else
                    parseClasspath( path.getClasspath() );
            }
        }
        else
        {
            System.err.println("Classpath element '" + id + "' not found.");
        }
    }

    /**
     * Parses an Arguments element.
     *
     * @param id - The Arguments id to parse.
     */
    private void
    parseArguments(String id)
    {
        java.util.List elements = null;
        Arguments args = null;
        Argument arg = null;

        try
        {
            args = (Arguments)objects_.get(id);
        }
        catch (ClassCastException e1)
        {
            System.err.println("'" + id + "' is not a Arguments element.");
        }

        if (args != null)
        {
            elements = args.getArgumentList();
            for(Iterator i = elements.iterator() ; i.hasNext() ; )
            {
                arg = (Argument)i.next();
                if (arg.getValue() != null)
                   arguments_.add( replaceTag(arg.getValue()) );
                else
                    parseArguments( arg.getArguments() );
            }
        }
        else
        {
            System.err.println("Arguments element '" + id + "' not found.");
        }
    }

    /**
     * Parses a Properties element.
     *
     * @param id - The Properties id to parse.
     */
    private void
    parseProperties(String id)
    {
        java.util.List elements = null;
        Properties props = null;
        Property prop = null;

        try
        {
            props = (Properties)objects_.get(id);
        }
        catch (ClassCastException e1)
        {
            System.err.println("'" + id + "' is not a Properties element.");
        }

        if (props != null)
        {
            elements = props.getPropertyList();
            for(Iterator i = elements.iterator() ; i.hasNext() ; )
            {
                prop = (Property)i.next();
                if ((prop.getName() != null) && (prop.getValue() != null))
                    properties_.setProperty(prop.getName(), replaceTag(prop.getValue()));
                else
                    parseProperties( prop.getProperties() );
            }
        }
        else
        {
            System.err.println("Properties element '" + id + "' not found.");
        }
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Parses the properties file.
     *
     * @param file The property file containing the configuration.
     *
     * @return The status (true if ok, else false).
     */
    public boolean
    parseFile(String file)
    {
        LauncherConfig config;

        if(file != null){
            try{
                config = LauncherConfigUnmarshaller.unmarshal(new java.io.File(file));
            }catch (java.io.IOException e){
                System.err.print("[I/O Error] : ");
                e.printStackTrace();
                return false;
            }
        }
        else
        {
            System.err.println("The file parameter is null ! ");
            return false;
        }

        parseConfiguration(config);
        return true;
    }

    /**
     * Set the run id and update variables.
     *
     * @param id - The run configuration to set.
     *
     * @return The status. True if ok, else false.
     */
    public boolean
    setRunId(String run_id)
    {
        Run run = null;
        java.util.StringTokenizer st = null;

        try
        {
            run = (Run)objects_.get(run_id);
        }
        catch (ClassCastException e1)
        {
            System.err.println("'" + run_id + "' is not a Run element.");
            return false;
        }
        catch (Exception e2)
        {
            System.err.println("Error when trying retrieve the Run element.");
            e2.printStackTrace();
            return false;
        }


        if (run != null)
        {
            className_ = run.getMainclassname();
            if (run.getClasspath() != null)
            {
                st = new java.util.StringTokenizer(run.getClasspath());
                while (st.hasMoreTokens())
                {
                    parseClasspath( st.nextToken() );
                }
            }
            if (run.getArguments() != null)
            {
                st = new java.util.StringTokenizer(run.getArguments());
                while (st.hasMoreTokens())
                {
                    parseArguments( st.nextToken() );
                }
            }
            if ( run.getProperties() != null)
            {
                st = new java.util.StringTokenizer(run.getProperties());
                while (st.hasMoreTokens())
                {
                    parseProperties( st.nextToken() );
                }
            }
        }
        else
        {
            System.err.println("Run element '" + run_id + "' not found.");
            return false;
        }
        return true;
    }

    /**
     * Get the class to launch.
     *
     * @return The class name.
     */
    public String
    getMainClassName()
    {
        return className_;
    }

    /**
     * Get the url list.
     *
     * @return An array of urls.
     */
    public String[]
    getUrls()
    {
        return (String[])urls_.toArray(new String[0]);
    }

    /**
     * Get the arguments list.
     *
     * @return An array of arguments.
     */
    public String[]
    getArguments()
    {
        return (String[])arguments_.toArray(new String[0]);
    }

    /**
     * Get properties.
     *
     * @return A Properties object containing all properties.
     */
    public java.util.Properties
    getProperties()
    {
        return properties_;
    }
}
