/*====================================================================

ObjectWeb Util Launcher Package.
Copyright (C) 2003 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): .

====================================================================*/

package org.objectweb.util.launcher.cmdline ;


import java.util.Vector;

import java.util.regex.Matcher ;
import java.util.regex.Pattern ;

import org.objectweb.util.cmdline.lib.DefaultRegExOptionArgument;
import org.objectweb.util.cmdline.api.Iterator;

import org.objectweb.util.launcher.CommandJava;
import org.objectweb.util.launcher.CommandFactory;
import org.objectweb.util.launcher.XBootClassLoader;
import org.objectweb.util.launcher.StringList;
import org.objectweb.util.launcher.TraceSystem;

import org.objectweb.util.launcher.parser.Parser;
import org.objectweb.util.launcher.parser.RunDescription;
import org.objectweb.util.launcher.parser.OptionDescription;


/**
 * Definition of the --runid option<BR>
 * <p>
 * This option specify the run identifier to use for starting the 
 * application
 * </p>
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 *
 * @version 0.1
 */
public class OptionLoad
    extends DefaultRegExOptionArgument
    implements OptionLauncher
{
    public final static String shortTag = "-load" ;
    public final static String longTag = "--loadid" ;

    public final static String separator = ":" ;
    public final static String list = ",";

    protected Parser parser_ ;
    protected Vector runs_ ;

    /**
     * Default Constructor
     */
    public OptionLoad(Parser p){
        super(new String[]{shortTag, longTag},
              "<RunID"+separator+
              "Option1"+list+"..."+list+"OptionX"+separator+
              "Arg1"+list+"..."+list+"ArgX"+separator+
              "Prop1"+list+"..."+list+"PropX>",
              "The identifier of the run to launch",
              "default"+separator+separator+separator,
              "([^:]+)"+separator+"([^:]*)"+separator+"([^:]*)"+separator+"(.*)");
        parser_=p;
        runs_ = new Vector();
    }

    
    protected class RunBuilder {
        protected String argument_ ;

        public RunBuilder(String arg) {
            this.argument_ = arg ;
        }
        
        protected void completeRun(CommandJava cmd,String run) {
            cmd.setName(run);
            TraceSystem.get("launcher").info("Completing java command " + cmd.getName());
            RunDescription desc = OptionLoad.this.parser_.getRun(cmd.getName());
            cmd.setClassname(desc.getMainclass());
            try {
                cmd.getLoader().addURL(desc.getClasses().toArray());
            } catch (java.net.MalformedURLException ex) {
                TraceSystem.get("launcher").error("Exception raised: " + ex);
            }
            cmd.addArguments(desc.getArguments());
            cmd.addProperties(desc.getProperties());
        }
        
        protected void completeOption(CommandJava cmd, String opts) {
            String[] opt = Pattern.compile(list).split(opts);
            for (int i=0 ; i<opt.length ; i++){
                if (!opt[i].equals("")) {
                    OptionDescription desc = OptionLoad.this.parser_.getOption(opt[i]);
                    try {
                        cmd.getLoader().addURL(desc.getClasses().toArray());
                    } catch (java.net.MalformedURLException ex) {
                        TraceSystem.get("launcher").error("Exception raised: " + ex);
                    }
                    cmd.addArguments(desc.getArguments());
                    cmd.addProperties(desc.getProperties());                
                }
            }
        }
        
        protected void completeArg(CommandJava cmd, String args) {
            String[] arg = Pattern.compile(list).split(args);
            StringList list = new StringList();
            for (int i=0 ; i<arg.length ; i++){
                if (!arg[i].equals("")) {
                    list.add(arg[i]);
                }
            }
            cmd.addArguments(list);
        }
        
        protected void completeProperties(CommandJava cmd, String props) {
            String[] prop = Pattern.compile(list).split(props);
            java.util.Properties list = new java.util.Properties();
            for (int i=0 ; i<prop.length ; i++){
                if (!prop[i].equals("")) {
                    String[] p = Pattern.compile("=").split(prop[i]);
                    if (p.length==2)
                        list.put(p[0],p[1]);
                    else if (p.length==1)
                        list.put(p[0],"");
                }
            }
            cmd.addProperties(list);
        }
        
        /**
         * Complete the command java by setting the identifier of
         * the run to use
         *
         * @param cmd - the command to complete
         */
        public CommandJava complete(){
            CommandJava cmd = CommandFactory.instance().create();
            Matcher match = getPattern().matcher(argument_);        
            match.matches();

            completeProperties(cmd,match.group(4));
            completeOption(cmd,match.group(2));
            completeRun(cmd,match.group(1));
            completeArg(cmd,match.group(3));
            return cmd;
        }
    }


    // ==================================================================
    //
    // Public methods for interface org.objectweb.util.cmdline.api.Option
    //
    // ==================================================================

    /**
     * Consumes command line arguments from an iterator.
     *
     * @param iterator The command line argument iterator.
     */
    public void
    consume(org.objectweb.util.cmdline.api.Iterator iterator)
    {
        setArgument(consumeArgument(iterator));
        runs_.addElement(new RunBuilder(getArgument()));
    }


    
    /**
     * Complete the command java by setting the identifier of
     * the run to use
     *
     * @param cmd - the command to complete
     */
    public CommandJava[] complete() {
        java.util.Iterator runs = runs_.iterator();
        Vector list = new Vector();
        while (runs.hasNext())
            list.addElement(((RunBuilder)runs.next()).complete());           
        return (CommandJava[])list.toArray(new CommandJava[0]);
    }


    public static String create(String run, String opt, String arg, String prop) {
        return shortTag+" "+run+separator+opt+separator+arg+separator+prop+" " ; 
    }
}
