/*====================================================================

ObjectWeb Util Ref Package.
Copyright (C) 2003 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.ref.lib;

// Package dependencies.
import org.objectweb.util.ref.api.Reference;
import org.objectweb.util.ref.api.Referencable;

/**
 * Base implementation of Reference objects.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public class ReferenceBase
  implements Reference
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The owner of the reference. */
    private Referencable owner_;
    
    /** The target of the reference. */
    private Referencable target_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    ReferenceBase()
    {
        // Inits internal state.
        owner_ = null;
        target_ = null;
    }

    /**
     * The constructor with only the owner of the reference.
     *
     * @param owner The owner of the reference.
     */
    public
    ReferenceBase(Referencable owner)
    {
        // Inits internal state.
        owner_ = owner;
        target_ = null;
    }

    /**
     * The constructor with both the owner and the target of the reference.
     *
     * @param owner The owner of the reference.
     * @param target The target of the reference.
     */
    public
    ReferenceBase(Referencable owner,
                  Referencable target)
    {
        // Inits internal state.
        owner_ = owner;
        target_ = null;
        setTarget(target);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for interface org.objectweb.util.ref.api.Reference
    //
    // ==================================================================

    /**
     * Obtains the owner of the reference.
     *
     * @return The owner of the reference.
     */
    public Referencable
    getOwner()
    {
        return owner_;
    }

    /**
     * Sets the owner of the reference.
     *
     * @param ref The owner of the reference.
     */
    public void
    setOwner(Referencable ref)
    {
        owner_ = ref;
    }

    /**
     * Obtains the target of the reference.
     *
     * @return The target of the reference.
     */
    public Referencable
    getTarget()
    {
        return target_;
    }

    /**
     * Sets the target of the reference.
     *
     * @param ref The target of the reference.
     */
    public void
    setTarget(Referencable ref)
    {
        // Does nothing if there is no change.
        if (ref == target_) return;
        
        // Cuts the previous dependency.
        cutIt();

        // Creates the new dependency.
        target_ = ref;
        target_.addReferencedBy(owner_);
    }

    /**
     * Cuts the reference.
     */
    public void
    cutIt()
    {
        if (target_ != null) {
            // Cuts the previous dependency.
            target_.removeReferencedBy(owner_);
            target_ = null;
        }
    }
}
