/*====================================================================

ObjectWeb Util Launcher Package.
Copyright (C) 2003 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Christophe Demarey.
Contributor(s): Romain Rouvoy.

====================================================================*/

package org.objectweb.util.launcher;


import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.net.MalformedURLException ;


/**
 * TestSuite for the XBootClassLoader
 *
 * <p>
 * The test suite is composed of:
 * <ol>
 * <li>A test case for the java.net.MalformedURLException</li>
 * <li>A test case for adding an URL in the ClassLoader</li>
 * <li>A test case for verifying redundancy in a ClassLoader</li>
 * </ol>
 * </p>
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 *
 * @version 0.1
 */
public class XBootClassLoaderTest
    extends TestSuite
{
    /**
     * Main Test Case for the XBootClassLoader
     *
     * @version 0.1
     */
    protected class XBootClassLoaderTestCase
        extends TestCase
    {
        /** the ClassLoader invariant */
        protected XBootClassLoader empty_ ;
        /** the ClassLoader fixture */
        protected XBootClassLoader full_ ;
        
        /** 
         * Dynamic call constructor 
         *
         * @param method - the Test Cas method to call
         */
        public XBootClassLoaderTestCase(String method) { super(method); }
        
        /**
         * The fixture intialisation method
         */
        protected void setUp() {
            this.empty_ = new XBootClassLoader() ;
            this.full_ = new XBootClassLoader() ;
        }

        /**
         * The fixture cleaning method
         */
        protected void tearDown() {
            this.empty_ = null ;
            this.full_ = null ;
        }
        
        /**
         * Test Case for the java.net.MalformedURLException
         *
         * <b>Verification: </b>
         * Malformed URLS are detected when added in the XBootClassLoader.
         *
         * <p>
         * Adds an url with an invalid syntax (no prefix) and waits 
         * for the exception.
         * </p>
         */
        public void testMalformedURL() {
            assertEquals("Initial comparison:",
                         empty_,
                         full_);
            try {
                full_.addURL("HelloWorld.jar");
                fail("The java.net.MalformedURLException must be thrown");
            } catch(Exception ex) {
                assertEquals("Class Type Assertion:",
                              "java.net.MalformedURLException",
                              ex.getClass().getName());
            } // the full_ ClassLoader must stay in a consistant state
            assertEquals("Final comparison: ",
                         empty_,
                         full_);
        }

        /**
         * Test Case for adding a valid URL on an invalid resource
         *
         * <b>Verification: </b>
         * URLs are accepted by the URLClassLoader.
         *
         * <p>
         * Adds an url with a valid syntax (using file: prefix) but 
         * an invalid resource location and check if the classloader 
         * inner state changed
         * </p>
         */
        public void testWellformedURL() {
            assertEquals("Initial comparison:",
                         empty_,
                         full_);
            try {
                full_.addURL("file:HelloWorld.jar");
                fail("java.io.IOException must be thrown");
            } catch(Exception ex) {
            }
            assertTrue("Final comparison:",
                       empty_.equals(full_));
        }

        /**
         * Test Case for adding urls pointing on the same resource 
         * HelloWorld.jar
         *
         * <b>Verification: </b>
         * URLs pointing on the same resource are considered only once by
         * the ClassLoader.
         *
         * <p>
         * ClassLoader are initialized with the file:HelloWorld.jar url.
         * Next adds the file:HelloWorld.jar url to the fixture.
         * </p>
         */
        public void testAddSameURL() {
            assertEquals("Initial comparison:",
                         empty_,
                         full_);
            try {
                empty_.addURL("file:HelloWorld.jar");
                full_.addURL("file:HelloWorld.jar");
            } catch(Exception ex) {
//                 assertEquals("Class Type Assertion:",
//                               "java.net.MalformedURLException",
//                               ex.getClass().getName());
            } // the full_ ClassLoader must stay in a consistant state
            assertEquals("Middle comparison:",
                         empty_,
                         full_);
            try {
                full_.addURL("file:HelloWorld.jar");
            } catch(Exception ex) {
                assertEquals("Class Type Assertion:",
                              "java.net.MalformedURLException",
                              ex.getClass().getName());
            }
            assertEquals("Final comparison: ",
                         empty_,
                         full_);
        }
    }


    
    /**
     * Default Constructor including 3 test cases
     */
    public XBootClassLoaderTest() {
        addTest(new XBootClassLoaderTestCase("testMalformedURL"));
        addTest(new XBootClassLoaderTestCase("testWellformedURL"));
//         addTest(new XBootClassLoaderTestCase("testAddSameURL"));
    }

    /**
     * Provide an instance of the test suite to use for testing the XBootClassLoader
     *
     * @return the XBootClassLoader Test Suite
     */
    public static Test suite() {        
        return new XBootClassLoaderTest();
    }
}
